; -----------------------------------------------------------------------------
;	MSX ROM sample program
; =============================================================================
;	2019/07/06	t.hara
; -----------------------------------------------------------------------------

		include		"msx.asm"

		org			0x4000
		rom_header	entry_point

; -----------------------------------------------------------------------------
;	entry point
; -----------------------------------------------------------------------------
entry_point::
		ld			sp, [himem]
		call		initialize
		call		main
		jr			entry_point

; -----------------------------------------------------------------------------
;	initialize
; -----------------------------------------------------------------------------
		scope		sc_initialize
initialize::
		;  SCREEN1
		ld			a, 1
		call		CHGMOD

		;	Initialize work area
		xor			a, a
		ld			[sec_counter], a
		ld			[free_counter + 0], a
		ld			[free_counter + 1], a

		;	Draw "TIMER = "
		ld			hl, timer_eq
		ld			de, SC1_PAT_NAME
		ld			bc, timer_eq_end - timer_eq
		call		LDIRVM

		;	Update timer hook
		di
		ld			hl, h_timi
		ld			de, old_h_timi
		ld			bc, 5
		ldir

		ld			hl, new_h_timi
		ld			de, h_timi
		ld			bc, 3
		ldir
		ei

		ret
timer_eq:
		defs		"TIMER = "
timer_eq_end:
		endscope

; -----------------------------------------------------------------------------
;	main
; -----------------------------------------------------------------------------
		scope		sc_main
main::
		;	Wait interrupt
		halt

		;	update counter string
		ld			hl, [free_counter]

		ld			a, 0xF0
		and			a, h
		rrca
		rrca
		rrca
		rrca
		call		get_char
		ld			[counter_string + 0], a

		ld			a, 0x0F
		and			a, h
		call		get_char
		ld			[counter_string + 1], a

		ld			a, 0xF0
		and			a, l
		rrca
		rrca
		rrca
		rrca
		call		get_char
		ld			[counter_string + 2], a

		ld			a, 0x0F
		and			a, l
		call		get_char
		ld			[counter_string + 3], a

		;	Draw counter
		ld			hl, counter_string
		ld			de, SC1_PAT_NAME + 8
		ld			bc, 4
		call		LDIRVM

		jp			main
		endscope

; -----------------------------------------------------------------------------
;	get_char
;	a <-- counter_char[ a ]
;	break: de
; -----------------------------------------------------------------------------
		scope		sc_get_char
get_char::
		ld			de, counter_char
		add			a, e
		ld			e, a
		xor			a, a
		adc			a, d
		ld			d, a
		ld			a, [de]
		ret
counter_char:
		defs		"0123456789ABCDEF"
		endscope

; -----------------------------------------------------------------------------
;	interrupt handler
; -----------------------------------------------------------------------------
		scope		sc_interrupt_handler
interrupt_handler::
		push		hl
		push		af

		ld			a, [sec_counter]
		inc			a
		ld			[sec_counter], a
		cp			a, 60
		jp			nz, exit

		xor			a, a
		ld			[sec_counter], a

		ld			hl, [free_counter]
		inc			hl
		ld			[free_counter], hl
	exit:
		pop			af
		pop			hl
		jp			old_h_timi
		endscope

; -----------------------------------------------------------------------------
;	VSYNC interrupt hook
; -----------------------------------------------------------------------------
new_h_timi::
		jp			interrupt_handler

; -----------------------------------------------------------------------------
;	PADDING
; -----------------------------------------------------------------------------
		rom_align

; -----------------------------------------------------------------------------
;	work area
; -----------------------------------------------------------------------------
old_h_timi			:= 0xC000
sec_counter			:= old_h_timi + 5
free_counter		:= sec_counter + 1
counter_string		:= free_counter + 2
